<?xml version="1.0"?>
<!-- .......................................................................

				     XLink2HTML
				     **********
     Description: XLink2HTML is a set of reusable XSLT templates for
     the transformation of XLink elements into HTML compatible with
     today's browsers. 

     .......................................................................
     Author: Fabio Arciniegas A.                           fabio@viaduct.com
     .......................................................................
     $Id$
 -->

<xsl:stylesheet version="1.0"
	       xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	       xmlns:xlink="http://www.w3.org/1999/xlink">


<!-- Name: xlink2html-extended
     **********
     This is the main template. It turns the specified node list of extended
     links into HTML. 

     parameters 
     **********
     extendedLinks: A node list with all the extended link elements that
                    the caller wishes to transform into HTML. 
                    Default value: All the children with xlink:type='extended'
--> 

<xsl:template name="xlink2html-extended">
     <xsl:param name="extendedLinks" select="*[@xlink:type='extended']"/>
     <xsl:for-each select="$extendedLinks">
	<xsl:for-each select="*[@xlink:type='arc']">
         
         <!-- Each arc may define a one to many or a many to many relationship
         (many to one relationships not considered relevant for this type
         of HTML representation
         -->

          <xsl:variable name="sourceRoleName" select="@xlink:from"/>
          <xsl:variable name="targetRoleName" select="@xlink:to"/>
          <xsl:choose> 
           <xsl:when test="count(../*[@xlink:role=$sourceRoleName]) &lt; 2">
             <xsl:call-template name="xlink2html-one-to-many">
               <xsl:with-param name="sourceRole" select="$sourceRoleName"/>
               <xsl:with-param name="targetRole" select="$targetRoleName"/>
             </xsl:call-template>
           </xsl:when>
           <xsl:otherwise>
             <xsl:call-template name="xlink2html-many-to-many">
                <xsl:with-param name="sourceRole">
                  <xsl:value-of select="@xlink:from"/>
                </xsl:with-param>
                <xsl:with-param name="targetRole">
                   <xsl:value-of select="@xlink:to"/>
                </xsl:with-param>
             </xsl:call-template>
           </xsl:otherwise>
           </xsl:choose>
          </xsl:for-each>
     </xsl:for-each>
</xsl:template>

<!-- ********************************************************************** -->
<!-- ********************************************************************** -->
<!-- Name: xlink2html-one-to-many
     **********
     This template contains the logic for one-to-many arcs. This is perhaps the
     most useful feature of Xlink2HTML. It creates the source representation
     from its content (or the xlink:title attribute/element if no content is provided)
     and the targets is created as a layer. The targets layer is shown when the
     target is clicked.

     parameters 
     **********
     sourceNode: the source node (unique) for this link
     targetRole: the name of the target role for this arc
--> 

<xsl:template name="xlink2html-one-to-many">
     <!-- Parameters -->
     <xsl:param name="sourceRole" select="undef"/>
     <xsl:param name="targetRole" select="undef"/>

     <xsl:variable name="header">
        &lt;table cellpadding=3 border=1 bgcolor=#FBFAE1>
     </xsl:variable> 
     <xsl:variable name="footer">
       &lt;/table>
     </xsl:variable>

     <xsl:element name="a">
        <xsl:attribute name="href">javascript:displayOptions(' 
          <xsl:value-of select="$header"/><xsl:text>&lt;td></xsl:text>
          <xsl:call-template name="xlink2html-create-list">
              <xsl:with-param name="nodes" select="../*[@xlink:role=$targetRole]"/>
          </xsl:call-template>
          <xsl:text>&lt;/td></xsl:text><xsl:value-of select="$footer"/>
        ');
        </xsl:attribute>
        <xsl:call-template name="xlink2html-simple-link-content">
            <xsl:with-param name="nodes" select="../*[@xlink:role=$sourceRole]"/>
        </xsl:call-template>
     </xsl:element>
     <br/>
</xsl:template>


<!-- ********************************************************************** -->
<!-- ********************************************************************** -->
<!-- Name: xlink2html-many-to-many
     **********
     This template contains the logic for many-to-many arcs. It creates the list
     of sources and  targets as layers to be shown when the appropriate buttons
     are called. 

     parameters 
     **********
     source-role: the name of the source role for this arc
     target-role: the name of the target role for this arc
--> 

<xsl:template name="xlink2html-many-to-many">
     <!-- Parameters -->
     <xsl:param name="sourceRole" select="undef"/>
     <xsl:param name="targetRole" select="undef"/>

     <xsl:variable name="header">
        &lt;table cellpadding=3 border=1 bgcolor=#FBFAE1>
     </xsl:variable> 
     <xsl:variable name="footer">
       &lt;/table>
     </xsl:variable>
     <xsl:element name="a">
        <xsl:attribute name="href">javascript:displayOptions(' 
          <xsl:value-of select="$header"/><xsl:text>&lt;td>There are </xsl:text>
           <xsl:value-of select="count(../*[@xlink:role=$sourceRole])"/><xsl:text> </xsl:text>
           <xsl:value-of select="$sourceRole"/><xsl:text> and </xsl:text>
           <xsl:value-of select="count(../*[@xlink:role=$targetRole])"/><xsl:text> </xsl:text>
           <xsl:value-of select="$targetRole"/>
           <xsl:text> involved in this link &lt;br/> </xsl:text> 
           <xsl:text>Please use the arrows on the left to see the participants&lt;br/> 
           </xsl:text>
          <xsl:value-of select="$footer"/>	 
        ');
        </xsl:attribute>
        <xsl:attribute name="onMouseOut">javascript:hideLayer();
        </xsl:attribute>
        [From  
        <xsl:value-of select="$sourceRole"/> 
        to 
        <xsl:value-of select="$targetRole"/>]
     </xsl:element>
     <xsl:element name="a">
        <xsl:attribute name="href">javascript:displayOptions('
          <xsl:value-of select="$header"/>
             <xsl:call-template name="xlink2html-create-list">
               <xsl:with-param name="nodes" select="../*[@xlink:role=$sourceRole]"/>
             </xsl:call-template>
          <xsl:text>&lt;/td></xsl:text>
          <xsl:value-of select="$footer"/>
        ');
        </xsl:attribute>
     <xsl:element name="img">
        <xsl:attribute name="border">0</xsl:attribute>
        <xsl:attribute name="src">xlink2html-down.gif</xsl:attribute>
     </xsl:element>
     </xsl:element>
     <xsl:element name="a">
        <xsl:attribute name="href">javascript:displayOptions('
          <xsl:value-of select="$header"/>
             <xsl:call-template name="xlink2html-create-list">
               <xsl:with-param name="nodes" select="../*[@xlink:role=$targetRole]"/>
             </xsl:call-template>
          <xsl:value-of select="$footer"/>
        ');
        </xsl:attribute>
     <xsl:element name="img">
        <xsl:attribute name="border">0</xsl:attribute>
        <xsl:attribute name="src">xlink2html-down.gif</xsl:attribute>
     </xsl:element>
     </xsl:element>
     <br/>
</xsl:template>




<!-- ********************************************************************** -->
<!-- ********************************************************************** -->
<!-- Name: xlink2html-create-list
     *******************
     Creates html content from a collection of locators and/or a resources. If
     the element has content, then the transformation of the content is
     inserted, if not, the title 
-->
<xsl:template name="xlink2html-create-list">
    <xsl:param name="nodes" select="/.."/>
    <xsl:for-each select="$nodes">
      <xsl:text>&lt;tr>&lt;td></xsl:text>
       <xsl:if test="@xlink:type='locator'">   
         <xsl:text>&lt;a href="</xsl:text><xsl:value-of select="@xlink:href"/>
         <xsl:text>"></xsl:text>
          <xsl:choose>
            <xsl:when test="xlink:title">   
                <xsl:value-of select="xlink:title"/><br/>
            </xsl:when>
            <xsl:when test="*|text()">   
                <xsl:apply-templates/>
            </xsl:when>
            <xsl:when test="@xlink:title">   
                <xsl:value-of select="@xlink:title"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="@xlink:href"/>
            </xsl:otherwise>
          </xsl:choose>
	  <xsl:text>&lt;/a></xsl:text>
        </xsl:if>
       <xsl:if test="@xlink:type='resource'">   
          <xsl:choose>
            <xsl:when test="*|text()">   
                <xsl:apply-templates/>
            </xsl:when>
            <xsl:when test="xlink:title">   
                <xsl:value-of select="xlink:title"/><br/>
            </xsl:when>
            <xsl:when test="@xlink:title">   
                <xsl:value-of select="@xlink:title"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:text>Resource must have a description!</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:if>
     <xsl:text>&lt;/td>&lt;/tr></xsl:text>
    </xsl:for-each>
</xsl:template>

<!-- ********************************************************************** -->
<!-- ********************************************************************** -->
<!-- Name: xlink2html-simple-link-content
     *******************
     Creates html content from a collection of locators and/or a resources. If
     the element has content, then the transformation of the content is
     inserted, if not, the title 
-->
<xsl:template name="xlink2html-simple-link-content">
    <xsl:param name="nodes" select="/.."/>
    <xsl:for-each select="$nodes">
       <xsl:if test="@xlink:type='resource'">   
          <xsl:choose>
            <xsl:when test="*|text()">   
                <xsl:apply-templates/>
            </xsl:when>
            <xsl:when test="xlink:title">   
                <xsl:value-of select="xlink:title"/><br/>
            </xsl:when>
            <xsl:when test="@xlink:title">   
                <xsl:value-of select="@xlink:title"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:text>Resource must have a description!</xsl:text>
            </xsl:otherwise>
          </xsl:choose>
        </xsl:if>
    </xsl:for-each>
</xsl:template>


<!-- ********************************************************************** -->
<!-- ********************************************************************** -->
<!-- Name: xlink2html-layer
     *******************
     Inserts the necessary hidden layer as part of the document
-->
<xsl:template name="xlink2html-layer">
   <div id="optionsLayer" style="position:absolute; visibility:show; left:25px; top:-50px; z-index:2">layer hidden off the screen</div>  
</xsl:template>



<!-- ********************************************************************** -->
<!-- ********************************************************************** -->
<!-- Name: xlink2html-onload
     *******************
     Inserts the necessary setup code into the onload attribute on the body tag.
-->
<xsl:template name="xlink2html-onload">
   <xsl:attribute name="onload">setupDescriptions()</xsl:attribute>
</xsl:template>


<!-- ********************************************************************** -->
<!-- ********************************************************************** -->
<!-- Name: xlink2html-js
     *******************
     Inserts the necessary JavaScript at the head of the page. Call this
     template when creating your HTML head element.
-->
<xsl:template name="xlink2html-js">
   <xsl:comment>
          XLink2HTML Javascript code. - Inserted by xlink2html xslt
          stylesheet. - This code allows the display of resources and locators
          within a floating layer.

          Contributions
          *************
          The browser identification portion of this script was implemented
          originally by Randy Bennett (rbennett@thezone.net) and acquired via
          javascript.com
   </xsl:comment>
   <xsl:element name="script">
       <xsl:attribute name="language">JavaScript</xsl:attribute>
       <xsl:comment>Begin
          // Browser identification
          function setupDescriptions() {
             var x = navigator.appVersion;
             y = x.substring(0,4);
             if (y>=4) setVariables();
          }
          var x,y,a,b;
          
          function setVariables(){
           if (navigator.appName == "Netscape") {
             h=".left=";
             v=".top=";
             dS="document.";
             sD="";
           }
           else 
           {
             h=".pixelLeft=";
             v=".pixelTop=";
             dS="";
             sD=".style";
           }
          }
          var isNav = (navigator.appName.indexOf("Netscape") !=-1);


          // Display Options
          // Displays a layer with arbitrary HTML on it. It is used 
          // to display the sources of a traversal.
          function displayOptions(content){
            if(isNav) {
              document.optionsLayer.document.write(content);
              document.optionsLayer.document.close();
              document.optionsLayer.left=x;
              document.optionsLayer.top=y;
           }
           else {
             optionsLayer.innerHTML=content;
             eval(dS+"optionsLayer"+sD+h+(x+15));
             eval(dS+"optionsLayer"+sD+v+y);
           }
        }

        function hideLayer(){
          if(isNav) 
            eval(document.optionsLayer.top=-50);
          else 
            optionsLayer.innerHTML="";
        }

        function handlerMM(e){
          x = (isNav) ? e.pageX : event.clientX;
          y = (isNav) ? e.pageY : event.clientY;
        }
        if (isNav){
          document.captureEvents(Event.MOUSEMOVE);
        }
        document.onmousemove = handlerMM;
//End  </xsl:comment>
   </xsl:element>
</xsl:template>

</xsl:stylesheet>




